%{
  #include <stdio.h>  /* For printf, etc. */
# include <stdlib.h> /* malloc. */
  int yyparse ();
# define YYDEBUG 1
  int yylex (void);
  void yyerror (char const *);
#include <benchmark/benchmark.h>
%}

%union {
  int val;
}
%token <val> NUM "number"
%token
  PLUS  "+"
  MINUS "-"
  STAR  "*"
  SLASH "/"
  LPAR  "("
  RPAR  ")"
%nterm  <val> exp

%left "+" "-"
%left "*" "/"

%%
exp
: exp "+" exp { $$ = $1 + $3; }
| exp "-" exp { $$ = $1 - $3; }
| exp "*" exp { $$ = $1 * $3; }
| exp "/" exp { $$ = $1 / $3; }
| "(" exp ")" { $$ = $2; }
| "number"    { $$ = $1; }
%%

const char* input;

int
yylex (void)
{
  int c = *input++;
  switch (c)
    {
    case '0':
    case '1':
    case '2':
    case '3':
    case '4':
    case '5':
    case '6':
    case '7':
    case '8':
    case '9':
      yylval.val = c - '0';
      return NUM;
    case '+': return PLUS;
    case '-': return MINUS;
    case '*': return STAR;
    case '/': return SLASH;
    case '(': return LPAR;
    case ')': return RPAR;
    case 0: return 0;
    default: return YYUNDEFTOK;
    }
}

void
yyerror (char const *s)
{
  fprintf (stderr, "%s\n", s);
}

const char *INPUT = "(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3+(1+2)*3";

#if 1
// Define another benchmark
static void BM_parse (benchmark::State& state)
{
  while (state.KeepRunning())
    {
      input = INPUT;
      yyparse ();
    }
}
BENCHMARK(BM_parse);

BENCHMARK_MAIN();

#else

int main(int argc, const char* argv[])
{
  if (1 < argc && strcmp (argv[1], "-p") == 0)
    yydebug = 1;
  input = INPUT;
  yyparse ();
}

#endif
